; ------------------------------------------------------------------------------------
; Remote SerialPort - Umsetzung per ECluster
; Client
; Author: Alexander Aigner
; PB 4.3+
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Internal
; ------------------------------------------------------------------------------------

XIncludeFile "Easy Cluster\Include\ECluster_inc.pbi"

Macro SP_IsInit() ; Initialisierungsmakro
  If Not Remote_SP_Master
    ProcedureReturn #Null
  EndIf
EndMacro

Structure Remote_SP_Data ; Data
  Server.i ; Slave
  Port.i ; Serial - Port
  Signal.i ; Signal-Semaphore
  Type.b ; Simple oder Mem
  Result1.i ; Results
  Result2.i
EndStructure

Enumeration
  #SP_SimpleResult
  #SP_MemResult
EndEnumeration

Global Remote_SP_Master
; Includepath, bei bedarf ndern
#ECluster_Path$ = "Easy Cluster\Include\Windows\ECluster.dll"

Procedure EventHandler(EMaster, Slave, Job, Event, *Mem, MemSize, UserData)
  ; Dummy
EndProcedure

; Jobhandler
Procedure SP_JobHandler(EMaster, Slave, Job, Event, *Mem, MemSize, *Port.Remote_SP_Data)
  With *Port
    If Event = #Event_Job_Finished
      Select \Type
        Case #SP_SimpleResult ; Einfache Jobs
          \Result1 = PeekL(*Mem)
          ECluster_FreeMemory(*Mem)
        Case #SP_MemResult ; Speicherrckgabe
          \Result1 = *Mem
          \Result2 = MemSize
      EndSelect
      SignalSemaphore(\Signal)
    EndIf
  EndWith
EndProcedure

; ------------------------------------------------------------------------------------
; Public
; ------------------------------------------------------------------------------------

Procedure InitRemoteSerialPort() ; Initialisieren
  Shared Remote_SP_Master
  Static isInit
  If Not isInit
    ECluster_Init(#ECluster_Path$)
    isInit = #True
  EndIf
  Remote_SP_Master = ECluster_Master_Init()
  ECluster_Master_SetCallback(Remote_SP_Master, @EventHandler(), #Null)
  ProcedureReturn Remote_SP_Master
EndProcedure

Procedure FreeRemoteSerialPort() ; Deinitialisieren
  Shared Remote_SP_Master
  If Remote_SP_Master
    ECluster_Master_DeInit(Remote_SP_Master)
    Remote_SP_Master = #Null
  EndIf
EndProcedure

; ------------------------------------------------------------------------------------
; Alle Funktionen wie die SerialPort() Funktionen, nur ConnectRemoteSerialPort()
; bentigt anstat eines #Port - Parameter einen Server und gibt das ntige Port -
; - Handle zurck
; ------------------------------------------------------------------------------------

Procedure ConnectRemoteSerialPort(Host$)
  SP_IsInit()
  ProcedureReturn ECluster_Master_AddSlave(Remote_SP_Master, Host$, 2020)
EndProcedure

Procedure AvailableRemoteSerialPortInput(*Port.Remote_SP_Data)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Master_DoJob(Remote_SP_Master, "AvailableSerialPortInput", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure AvailableRemoteSerialPortOutput(*Port.Remote_SP_Data)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Master_DoJob(Remote_SP_Master, "AvailableSerialPortOutput", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure GetRemoteSerialPortStatus(*Port.Remote_SP_Data, Attributes)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Parameter_AddLong(Paras, Attributes)
    ECluster_Master_DoJob(Remote_SP_Master, "GetSerialPortStatus", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure IsRemoteSerialPort(*Port.Remote_SP_Data)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Master_DoJob(Remote_SP_Master, "IsSerialPort", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure ReadRemoteSerialPortData(*Port.Remote_SP_Data, *Mem, MemSize)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_MemResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Parameter_AddLong(Paras, MemSize)
    ECluster_Master_DoJob(Remote_SP_Master, "ReadSerialPortData", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    CopyMemory(\Result1, *Mem, \Result2)
    ECluster_FreeMemory(\Result1)
    ProcedureReturn \Result2
  EndWith
EndProcedure

Procedure RemoteSerialPortError(*Port.Remote_SP_Data)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Master_DoJob(Remote_SP_Master, "SerialPortError", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure RemoteSerialPortTimeouts(*Port.Remote_SP_Data, RIT, RTTC, RTTM, WTTC, WTTM)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Parameter_AddLong(Paras, RIT)
    ECluster_Parameter_AddLong(Paras, RTTC)
    ECluster_Parameter_AddLong(Paras, RTTM)
    ECluster_Parameter_AddLong(Paras, WTTC)
    ECluster_Parameter_AddLong(Paras, WTTM)
    ECluster_Master_DoJob(Remote_SP_Master, "SerialPortTimeouts", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure SetRemoteSerialPortStatus(*Port.Remote_SP_Data, Attributes, Value)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Parameter_AddLong(Paras, Attributes)
    ECluster_Parameter_AddLong(Paras, Value)
    ECluster_Master_DoJob(Remote_SP_Master, "SetSerialPortStatus", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure WriteRemoteSerialPortData(*Port.Remote_SP_Data, *Mem, MemSize)
  SP_IsInit()
  Protected Paras, *Mem2
  With *Port
    *Mem2 = ECluster_AllocateMemory(MemSize)
    CopyMemory(*Mem, *Mem2, MemSize)
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Parameter_AddMem(Paras, *Mem2, MemSize)
    ECluster_Master_DoJob(Remote_SP_Master, "WriteSerialPortData", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure WriteRemoteSerialPortString(*Port.Remote_SP_Data, String$, Format = #PB_Ascii)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Parameter_AddString(Paras, String$)
    ECluster_Parameter_AddLong(Paras, Format)
    ECluster_Master_DoJob(Remote_SP_Master, "WriteSerialPortString", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    ProcedureReturn \Result1
  EndWith
EndProcedure

Procedure OpenRemoteSerialPort(Server, SerialPortName$, Bauds, Parity, pData, Stop, HandshakeMode, InputBufferSize, OutputBufferSize)
  SP_IsInit()
  Protected *Port.Remote_SP_Data = AllocateMemory(SizeOf(Remote_SP_Data)), Paras
  With *Port
    \Server = Server
    \Signal = CreateSemaphore()
    \Type = #SP_SimpleResult
    Paras = ECluster_Parameter_New()
    ECluster_Parameter_AddLong(Paras, #PB_Any)
    ECluster_Parameter_AddString(Paras, SerialPortName$)
    ECluster_Parameter_AddLong(Paras, Bauds)
    ECluster_Parameter_AddLong(Paras, Parity)
    ECluster_Parameter_AddLong(Paras, pData)
    ECluster_Parameter_AddLong(Paras, Stop)
    ECluster_Parameter_AddLong(Paras, HandshakeMode)
    ECluster_Parameter_AddLong(Paras, InputBufferSize)
    ECluster_Parameter_AddLong(Paras, OutputBufferSize)
    ECluster_Master_DoJob(Remote_SP_Master, "OpenSerialPort", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    \Port = \Result1
    If \Port
      ProcedureReturn *Port
    Else
      FreeSemaphore(\Signal)
      FreeMemory(*Port)
      ProcedureReturn #Null
    EndIf
  EndWith
EndProcedure

Procedure CloseRemoteSerialPort(*Port.Remote_SP_Data)
  SP_IsInit()
  Protected Paras
  With *Port
    Paras = ECluster_Parameter_New()
    \Type = #SP_SimpleResult
    ECluster_Parameter_AddLong(Paras, \Port)
    ECluster_Master_DoJob(Remote_SP_Master, "CloseSerialPort", \Server, Paras, @SP_JobHandler(), *Port)
    WaitSemaphore(\Signal)
    FreeSemaphore(\Signal)
    FreeMemory(*Port)
    ProcedureReturn #True
  EndWith
EndProcedure
; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 74
; FirstLine = 206
; Folding = ----
; EnableXP